

# http://www.physics.udel.edu/~watson/scen103/ascii.html
Enter = '\r'
Tab = '\t'
Ctrl_C = '\x03'
Ctrl_D = '\x04'
Ctrl_E = '\x05'
Ctrl_F = '\x06'
Ctrl_H = '\x08'
Ctrl_L = '\x0C'
Ctrl_P = '\x10'
Ctrl_U = '\x15'
Ctrl_X = '\x18'
Ctrl_Y = '\x19'
Ctrl_Z = '\x1A'
Star = '\x2A'
Up = '\x1b\x4fA'
Down = '\x1b\x4fB'
Right = '\x1b\x4fC'
Left = '\x1b\x4fD'

Space = ' '
QueryPost = 'Q'
Push = 'X'
GoMainMenu = ' ' + Left * 5
GoMainMenu_TypeQ = 'q' * 5
Refresh = Ctrl_L
ControlCode = Ctrl_U + Star
Backspace = Ctrl_H
